/*	VIRLIST V1.1
 *
 *  Read McAfee's VIRLIST.TXT file and display viruses
 *  with selected characteristics.
 *
 *	  Program:	VIRLIST Version 1.1
 *	  Platform:	MS-DOS/Turbo C V2.0
 *	  Author:	Jason Mathews
 *	  Date: 	6/12/92
 *
 *  Copyright (C) 1992 by Jason Mathews.  Permission is granted to any
 *  individual or institution to use, copy or redistribute this software so long
 *  as it is not sold for profit, provided this copyright notice is retained.
 *
 * History:
 *	 2/24/92 - Developed original program
 *	 6/12/92 - Added negative logic to filter out characteristics
 *		   Added logic for checking virus length (EQ, LT, GT)
 */

#include <stdio.h>
#include "virlist.h"

void main (int argc, char **argv)
{
	int matches = 0;
	FILE *fp = fopen (VIRLIST_FILE, "r");

	if (fp==0)
	{
		fprintf(stderr, "Cannot file virus list '%s'\n", VIRLIST_FILE);
		fputs("Please copy this file into the appropriate directory.\n", stderr);
		exit(1);
	}

	CheckArgs(argc, argv);	/* Check arguments */

	SkipHeader(fp); 	   /* Skip the virlist.txt header */

	while (fgets(buffer, 1024, fp))
	{
		if (*buffer == '\n') break;
			if (GetRecord(buffer) > 0)
			{
			 PrintRecord();
			 matches++;
			}
	}
	fclose(fp);

	printf("\nTotal of %d match%s found\n", matches, (matches!=1) ? "es" : "");

	Legend();			/* Show code legend and exit */
	exit(0);
}  /** main **/


void SetFlag (int field, int msg_num, char set_code)
{
	/*  set flag code to TRUE for '+' code
	 *  and FALSE for '-' code.
	 */
	flags[field-1] = (set_code == '+') ? TRUE : FALSE;
	printf("\t%c%s\n", set_code, messages[msg_num]);
}  /** SetFlag **/


/* CheckArgs:
 *  Check program flags & options
 */
void CheckArgs (int argc, char **argv)
{
    int i, opt, offset;
    bool valid;

    /* inititalize virus damage codes */
	boot = data = format = links = ops = program = 0;

    if (argc == 1) Help();

    puts("VIRLIST Program options:");
    for (i=1; i<argc; i++)
    {
	if (*argv[i] == '-' || *argv[i] == '+')
        {
		/* Check for missing argument */
		if (argv[i][1] == 0) Help();
		else if (argv[i][1] == 's')
		{
		switch (argv[i][2]) {
		 case 'E':
			size_code = EQUAL;
			break;
		 case 'L':
			size_code = LESS_THAN;
			break;
		 case 'G':
			size_code = GREATER_THAN;
			break;
		 case 'O':
			size_code = OVERWRITE;
			puts("\tmatch all overwriting viruses");
			break;
		 case 'V':
			size_code = VARIES;
			puts("\tmatch all varied-length viruses");
			break;
		 case 'S':
			size_code = SPAWN;
			puts("\tmatch all spawning viruses");
			break;
		 default:
			size_code = INVALID;
			fprintf(stderr, "Invalid virus size usage: '%s'\n", argv[i]);
			break;
		} /* switch on single letter code */

		if (size_code != INVALID && size_code > 0)
		{
			size_value = atol(argv[i]+4);
			printf("\tmatch virus size %s %ld bytes\n",
				size_strings[size_code-1], size_value);
		}
	} /* if '-s' argument */
	else  /* parse argument string */
	    for (offset=1; argv[i][offset] != 0; offset++)
	      switch (argv[i][offset]) /* check letter codes */
	      {
			case 'B': SetField (boot, 1);
			case 'C': SetField (program, 2);
			case 'D': SetField (data, 3);
			case 'F': SetField (format, 4);
			case 'L': SetField (links, 5);
			case 'O': SetField (ops, 6);

			case 'V':
				size_code = VARIES;
				printf("\t%s\n", messages[8]);
				break;

			case 'P': SetFlag (10, 7, *argv[i]);
				  break;

			case 'h':
			case '?': Help();

			default:
				valid = FALSE;
				for (opt='1'; opt<='9' ; opt++) /* check for options 1-9 */
				 if (argv[i][offset] == opt)
				 {
					SetFlag (opt-'0', opt-'0'+8, *argv[i]);
					valid = TRUE;
					break;
				 }
				if (valid == FALSE)
				   fprintf(stderr, "invalid option/flag: %c\n", argv[i][offset]);
				break;
	      } /* switch */
        } /* if */
    } /* for */

    puts(messages[0]);  /* show a dashed line */
}  /** CheckArgs **/


/* CheckCode:
 *  Check if code is set for entry and print its corresponding letter
 *  or a blank.
 */
void CheckCode (int code, char letter)
{
	/* first check for potential characteristics with (?) code */
	if (virus.codes[code-1] == '?') putchar('?');
	else putchar( (virus.codes[code-1]) ? letter : '.');

    putchar(' ');
}  /** CheckCode **/


/* GetRecord:
 *   Get fields from virlist entry and fill in record
 */
int GetRecord (char *buf)
{
    int i, length = strlen(buf);
    char tmp[20];			/* temp buffer */
    int match = 0;			/* number of matches */

    if (length < 58) return(0); /* entry too short - no codes to check */

	/* strip off virus name and disinfector */

    Copy(name, 26);
    buf += 26;
    Copy(disinfector, 9);
    buf += 13;

	/* Parse each virus code and check value for 'x', '?', or blank */

    for (i=0; i<10; i++)
    {
	if (*buf == '?')  /* take a '?' as maybe */
	{
		if (flags[i] != NOT_USED)
		{
			match++;
			virus.codes[i] = '?';
		}
	}
	else if (*buf == 'x')
        {
		virus.codes[i] = 1;
		if (flags[i]==TRUE) match++;
		else if (flags[i] == FALSE) return 0; /* no match */
        }
	else  /* must be a blank */
	{
		virus.codes[i] = 0;
		if (flags[i]==FALSE) match++;
		else if (flags[i] == TRUE) return 0; /* no match */
	}
	buf += 2;
    } /* for */

    /* check virus size */

    if (strncmp(buf, "Overwrite", 9) == 0)
		virus.size = OVERWRITE;
	else if (strstr(buf+2, "Varies"))
		virus.size = VARIES;
    else if (strncmp(buf+4, "N/A", 3) == 0 || strncmp(buf+1, "See Note", 8) == 0)
		virus.size = OTHER;
    else if (strstr(buf, "Spawning"))
		virus.size = SPAWN;
    else /* check for an actual number */
    {
		strcpy(tmp, buf, 7);
		tmp[7] = 0;
		virus.size = atol(tmp);
	}

    switch (size_code) {
	case OVERWRITE:
		if (virus.size == OVERWRITE) match++;
		else return(0); /* no match */
		break;
	case VARIES:
		if (virus.size == VARIES) match++;
		else return(0); /* no match */
		break;
	case SPAWN:
		if (virus.size == SPAWN) match++;
		else return(0);  /* no match */
		break;
		 case EQUAL:
			if (virus.size == size_value) match++;
			else return(0);
			break;
		 case GREATER_THAN:
			if (virus.size > size_value) match++;
			else return(0);
			break;
		 case LESS_THAN:
			if (virus.size < size_value && virus.size > 0) match++;
            else return(0);
			break;
    }

    buf += 11; /* skip over size field */

    /* reset damage flags for virlist entry */
    virus.boot = virus.data = virus.format = virus.links =
    virus.ops = virus.program = 0;

    /* Check damage fields */
    while (*buf != 0)
    {
        switch( *buf)
        {
          case 'B': Damage(boot);
          case 'D': Damage(data);
          case 'F': Damage(format);
          case 'L': Damage(links);
          case 'O': Damage(ops);
          case 'P': Damage(program);
        } /* switch */
        buf++;
    }

    return match;
}  /** GetRecord **/


/* Help:
 *   Show program options
 */
void Help()
{
    int i;
    puts("VIRLIST V1.1: By Jason Mathews (C) June 1992\n");
    puts("Usage:\n\tvirlist [opcode BCDFLOP1234567890] [-sSize_code Size]\n");
    puts("opcode = '+' for the characterisic to be true");
    puts("         '-' for it to be false.\n");
    puts("Options:");
    for (i=1; i < MSG_NUM; i++)
    {
	printf("\t%s\n", messages[i]);
	if (i==15)
	{
		fprintf(stderr, "press any key to continue");
		getch();
		fputc('\n', stderr);
	}
    }

    puts("\nSize codes:\n\tEQ, GT, LT comparison operators followed by a number");
    puts("\t    e.g. -sEQ1000 to show viruses with a length of 1000 bytes");
    puts("\tO = show overwriting viruses");
    puts("\tV = show varied length viruses");
    puts("\tS = show spawning viruses");
	puts("\n\nFor example: type 'VIRLIST -sLT1000 +2' to display viruses");
	puts("\t     that are less than 1000 bytes and use self-encryption");
    exit(1);
}  /** Help **/


/* Legend:
 *  Show code legend
 */
void Legend()
{
    int i;
    putchar('\n');
    puts(messages[0]);
    puts("Legend:");
    for (i=1; i < MSG_NUM; i++)
    {
        /* check for start of number codes to insert a new line */
        if (i==9) putchar('\n');
        printf("\t%s\n", messages[i]);
    }
}  /** Legend **/


/* PrintRecord:
 *  Print virlist record
 */
void PrintRecord()
{
    int i;
    printf("%s  ", virus.name);

	/*	Check if code is set and print letter if set or a blank if not
	 *	Note that the 10th code uses a 'P' instead of a number.
	 */
    for (i=1; i<=10; i++)
	CheckCode (i, (i<10) ? i+'0' : 'P');

    switch(virus.size)
    {
     case -1:
        printf("Overwrites");
        break;
     case -2:
        printf("  varies  ");
        break;
     case 0:
        printf("          ");
        break;
     case -3:
        printf("    N/A   ");
        break;
	case -4:
		printf("Spawning  ");
		break;
     default:
        printf("%7ld   ", virus.size);
    }

    if (virus.boot) printf(" B");
    if (virus.data) printf(" D");
    if (virus.format) printf(" F");
    if (virus.links) printf(" L");
    if (virus.ops) printf(" O");
    if (virus.program) printf(" C");
    putchar('\n');
}  /** PrintRecord **/


/* SkipHeader:
 *	Read past 'virlist.txt' header, which ends with a line of all dashes (-)
 *	The file location should now be at the beginning of the virlist entries.
 */
void SkipHeader (FILE *fp)
{
	while (fgets(buffer, 1024, fp))
	{
		if (strncmp(buffer, "------", 6) == 0) return;
	}
}  /** SkipHeader **/
